//
//  BleListVC.m
//  PrinterDemo
//
//  Created by Apple Mac mini intel on 2024/4/20.
//

#import "BleListVC.h"
#import "XBLEManager.h"
#import "UIView+Toast.h"

@interface BleListVC ()<UITableViewDelegate,UITableViewDataSource, XBLEManagerDelegate>
@property (weak, nonatomic) IBOutlet UITableView *bleTableView;
@property (nonatomic,strong) NSMutableArray *dataArr;
@property (nonatomic,strong) NSMutableArray *rssiList;
@property (strong, nonatomic) UIActivityIndicatorView *indicator;

@end

@implementation BleListVC

- (void)viewDidLoad {
    [super viewDidLoad];
    self.title = @"BLE Search";
    [XBLEManager sharedInstance].delegate = self;
    [[XBLEManager sharedInstance] startScan];
    _indicator = [[UIActivityIndicatorView alloc] initWithActivityIndicatorStyle:UIActivityIndicatorViewStyleGray];
    _indicator.center = self.view.center;
    [self.view addSubview:self.indicator];

}

- (void)viewDidDisappear:(BOOL)animated {
    [super viewDidDisappear:animated];
    [[XBLEManager sharedInstance] removeDelegate:self];
}

#pragma mark - XBLEManagerDelegate

- (void)xbleDiscoverPeripheral:(CBPeripheral *)peripheral advertisementData:(NSDictionary *)advertisementData RSSI:(NSNumber *)RSSI {
    
    if (![self.dataArr containsObject:peripheral]) {
        [self.dataArr addObject:peripheral];
    }
    
    if (![self.rssiList containsObject:RSSI]) {
        [self.rssiList addObject:RSSI];
    }
    
    [self.bleTableView reloadData];
}

- (void)xbleConnectPeripheral:(CBPeripheral *)peripheral {
    [self.indicator stopAnimating];
    [self.navigationController popViewControllerAnimated:YES];
}

- (void)xbleFailToConnectPeripheral:(CBPeripheral *)peripheral error:(NSError *)error {
    [self.indicator stopAnimating];
    [self.view makeToast:@"connect fail" duration:1.f position:CSToastPositionCenter];
}

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView {
    return 1;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    return self.dataArr.count;
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    static NSString *cellId = @"printerCell";
    UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:cellId];
    if (!cell) {
        cell = [[UITableViewCell alloc] initWithStyle:UITableViewCellStyleSubtitle reuseIdentifier:cellId];
    }
    cell.accessoryType = UITableViewCellAccessoryDisclosureIndicator;
    
    if (indexPath.row < self.dataArr.count) {
        CBPeripheral *peripheral = self.dataArr[indexPath.row];
        cell.textLabel.text = peripheral.name;
        if (peripheral.name.length == 0) {
            cell.textLabel.text = @"unknow";
        }
    }
    
    if (indexPath.row < self.rssiList.count) {
        NSNumber *rssi = self.rssiList[indexPath.row];
        cell.detailTextLabel.text = [NSString stringWithFormat:@"RSSI:%zd",rssi.integerValue];
    }
    return cell;
}

- (CGFloat)tableView:(UITableView *)tableView heightForRowAtIndexPath:(NSIndexPath *)indexPath {
    return 100;
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
    CBPeripheral *peripheral = self.dataArr[indexPath.row];
    [self.indicator startAnimating];
    if ([XBLEManager sharedInstance].isConnected) {
        [[XBLEManager sharedInstance] disconnectRootPeripheral];
    }
    [[XBLEManager sharedInstance] connectDevice:peripheral];
}

- (IBAction)scanAgain:(id)sender {
    [self.dataArr removeAllObjects];
    [[XBLEManager sharedInstance] startScan];
}

#pragma mark - lazy
- (NSMutableArray *)dataArr {
    if (!_dataArr) {
        _dataArr = [NSMutableArray array];
    }
    return _dataArr;
}

@end
