//
//  POSCompressionVC.m
//  PrinterDemo
//
//  Created by Apple Mac mini intel on 2024/5/30.
//

#import "POSCompressionVC.h"
#import "UIView+Toast.h"
#import "XSDKPrinter.h"

@interface POSCompressionVC ()<UIImagePickerControllerDelegate,UINavigationControllerDelegate>

@property (weak, nonatomic) IBOutlet UISegmentedControl *modeSeg;
@property (weak, nonatomic) IBOutlet UILabel *picStatusLabel;
@property (strong, nonatomic) UIImage *compressionImg;
@end

@implementation POSCompressionVC

- (void)viewDidLoad {
    [super viewDidLoad];
    self.title = @"POS Compression";
    self.compressionImg = [UIImage imageNamed:@"nv_test"];
}

- (IBAction)selectPicAction:(id)sender {
    UIImagePickerController * imagePicker = [[UIImagePickerController alloc] init];
    imagePicker.editing = YES;
    imagePicker.delegate = self;
    imagePicker.allowsEditing = YES;
    imagePicker.sourceType = UIImagePickerControllerSourceTypePhotoLibrary;
    [self presentViewController:imagePicker animated:YES completion:nil];
}

- (void)imagePickerController:(UIImagePickerController *)picker didFinishPickingMediaWithInfo:(NSDictionary<NSString *,id> *)info {
    
    [picker dismissViewControllerAnimated:YES completion:nil];
    UIImage *img = [info valueForKey:UIImagePickerControllerOriginalImage];
    
    if(CGImageGetWidth(img.CGImage) > 480) {
        img = [self imageWithscaleImage:img andScaleWidth:480 andScaleHeight:480*CGImageGetHeight(img.CGImage)/CGImageGetWidth(img.CGImage)];
    }
    
    if(img) {
        self.compressionImg = img;
        self.picStatusLabel.text = @"Ready";
        self.picStatusLabel.textColor = [UIColor greenColor];
    } else {
        self.compressionImg = [UIImage imageNamed:@"nv_test"];
        self.picStatusLabel.text = @"Default";
        self.picStatusLabel.textColor = [UIColor blackColor];
    }
}

-(UIImage *)imageWithscaleImage:(UIImage *)image andScaleWidth:(CGFloat)maxWidth andScaleHeight:(CGFloat)maxHeight {
    CGFloat width = image.size.width;
    CGFloat height = image.size.height;
    if (maxHeight == 0) {
        maxHeight = (int)(maxWidth * height / width);
    }
    
    if (maxWidth == 0) {
        maxWidth = (int)(maxHeight * width / height);
    }
    
    CGSize size = CGSizeMake(maxWidth, maxHeight);
    UIGraphicsBeginImageContext(size);
    [image drawInRect:CGRectMake(0, 0, maxWidth, maxHeight)];
    UIImage *resultImage = UIGraphicsGetImageFromCurrentImageContext();
    UIGraphicsEndImageContext();
    return resultImage;
}

- (IBAction)printAction:(id)sender {
    
    if (![XBLEManager sharedInstance].isConnected) {
        [self.view makeToast:@"ble no connect" duration:1.f position:CSToastPositionCenter];
        return;
    }
    
    XPOSCommand *command = [[XPOSCommand alloc] init];
    [command initializePrinter];
    [command selectAlignment:POSAlignmentLeft];
    
    if (self.modeSeg.selectedSegmentIndex == 0) {
        [command printImage:self.compressionImg type:BMPThreshold mode:BmpNormal];
    } else {
        [command compressionPrintImage:self.compressionImg type:BMPThreshold mode:BmpNormal];
    }
    
    [command printAndFeedLines:6];
    [command cutPaperWithMode:PaperCutModePartialCut];
    
    [self blePrintWithData:[command getCommand]];
}

- (void)blePrintWithData:(NSData *)data {
    NSUInteger maxWriteLength = [[XBLEManager sharedInstance].writePeripheral maximumWriteValueLengthForType:CBCharacteristicWriteWithoutResponse];
    if (data.length > maxWriteLength) {
        NSLog(@"Send in packets");
        NSUInteger offset = 0;
        while (offset < data.length) {
            NSUInteger chunkLength = MIN(maxWriteLength, data.length - offset);
            NSData *chunk = [data subdataWithRange:NSMakeRange(offset, chunkLength)];
            [[XBLEManager sharedInstance] writeCommandWithData:chunk];
            offset += chunkLength;
        }
    } else {
        NSLog(@"Direct Send");
        [[XBLEManager sharedInstance] writeCommandWithData:data];
    }
}

@end
