//
//  POSPrinterVC.m
//  PrinterDemo
//
//  Created by Apple Mac mini intel on 2024/5/4.
//

#import "POSPrinterVC.h"
#import "XSDKPrinter.h"
#import "UIView+Toast.h"

@interface POSPrinterVC ()<XBLEManagerDelegate, XWIFIManagerDelegate>
// bluetooth manager
@property (strong, nonatomic) XBLEManager *bleManager;
// wifi manager
@property (strong, nonatomic) XWIFIManager *wifiManager;
@end

@implementation POSPrinterVC

- (void)viewDidLoad {
    [super viewDidLoad];
    self.title = @"POS PRINT";
    _bleManager = [XBLEManager sharedInstance];
    _bleManager.delegate = self;
    _wifiManager = [XWIFIManager sharedInstance];
    _wifiManager.delegate = self;
}

- (IBAction)printTextAction:(id)sender {
    XPOSCommand *command = [[XPOSCommand alloc] init];
    [command initializePrinter];
    [command setHorizontalTab];
    [command addText:@"123456789abc\n"];
    [command selectTextSize:TextWidth1 | TextHeight1];
    [command addText:@"123456789abc\n"];
    [command selectTextSize:TextWidth2 | TextHeight2];
    [command addText:@"123456789abc\n"];
    [command selectTextSize:TextWidth3 | TextHeight3];
    [command addText:@"123456789abc\n"];
    [command printAndFeedLines:5];
    [self printWithData:[command getCommand]];
}

- (IBAction)printBarcodeAction:(id)sender {
    XPOSCommand *command = [[XPOSCommand alloc] init];
    [command initializePrinter];
    [command selectAlignment:POSAlignmentCenter];
    [command selectHRICharacterPrintPosition:2];
    [command selectBarcodeHeight:70];
    [command addText:@"UPC-A\n"];
    [command printBarcodeWithType:BarcodeTypeUPCA andContent:@"123456789012"];
    [command lineFeed];
    [command addText:@"UPC-E\n"];
    [command printBarcodeWithType:BarcodeTypeUPCE andContent:@"042100005264"];
    [command lineFeed];
    [command addText:@"JAN13\n"];
    [command printBarcodeWithType:BarcodeTypeJAN13_EAN13 andContent:@"123456791234"];
    [command lineFeed];
    [command addText:@"JAN8\n"];
    [command printBarcodeWithType:BarcodeTypeJAN8_EAN8 andContent:@"12345678"];
    [command lineFeed];
    [command addText:@"CODE39\n"];
    [command setBarcodeWidth:2];
    [command printBarcodeWithType:BarcodeTypeCode39 andContent:@"ABCDEFGHI"];
    [command lineFeed];
    [command addText:@"ITF\n"];
    [command printBarcodeWithType:BarcodeTypeITF andContent:@"123456789012"];
    [command lineFeed];
    [command addText:@"CODEBAR\n"];
    [command printBarcodeWithType:BarcodeTypeCodabar andContent:@"A37859B"];
    [command lineFeed];
    [command addText:@"CODE93\n"];
    [command printBarcodeWithType:BarcodeTypeCode93 andContent:@"123456789"];
    [command lineFeed];
    [command addText:@"CODE128\n"];
    [command printBarcodeWithType:BarcodeTypeCode128 andContent:@"CODE128 123456"];
    [command lineFeed];
    [command printAndFeedLines:5];
    [self printWithData:[command getCommand]];
}

- (IBAction)printQrcodeAction:(id)sender {
    XPOSCommand *command = [[XPOSCommand alloc] init];
    [command initializePrinter];
    [command selectAlignment:POSAlignmentCenter];
    [command printQRCodeWithContent:@"www.google.com" moduleSize:6 ecLevel:QRErrorCorrectionLevelL];
    [command lineFeed];
    [command printQRCodeWithContent:@"www.google.com" moduleSize:8 ecLevel:QRErrorCorrectionLevelM];
    [command lineFeed];
    [command printQRCodeWithContent:@"www.google.com" moduleSize:12 ecLevel:QRErrorCorrectionLevelQ];
    [command lineFeed];
    [command printQRCodeWithContent:@"www.google.com" moduleSize:16 ecLevel:QRErrorCorrectionLevelH];
    [command printAndFeedLines:6];
    [command cutPaperWithMode:PaperCutModePartialCut];
    [self printWithData:[command getCommand]];
}

- (IBAction)printImageAction:(id)sender {
    UIImage *image = [UIImage imageNamed:@"image"];
    XPOSCommand *command = [[XPOSCommand alloc] init];
    [command initializePrinter];
    [command selectAlignment:POSAlignmentLeft];
    [command printImage:image type:BMPThreshold mode:BmpNormal];
    [command printAndFeedLines:6];
    [command cutPaperWithMode:PaperCutModePartialCut];
    [self printWithData:[command getCommand]];
}

- (IBAction)downloadFlashAction:(id)sender {
    XPOSCommand *command = [[XPOSCommand alloc] init];
    NSMutableArray <UIImage *>*images = [NSMutableArray arrayWithArray:@[[UIImage imageNamed:@"Rabit"],[UIImage imageNamed:@"Logo"],[UIImage imageNamed:@"JsLogo"]]];
    [command definedFlashBmpWithImage:images];
    NSData *data = [command getCommand];
    if (data.length < 8096) {
        [self printWithData:data];
    } else {
        [self.view makeToast:@"Exceeded the flash download capacity" duration:1.f position:CSToastPositionCenter];
    }
}

- (IBAction)printFlashAction:(id)sender {
    XPOSCommand *command = [[XPOSCommand alloc] init];
    [command initializePrinter];
    [command addText:@"FLASH Bitmap serial number 1:\n"];
    [command printBitmapToFlashWithN:1 andM:0];
    [command lineFeed];
    [command addText:@"FLASH Bitmap serial number 2:\n"];
    [command printBitmapToFlashWithN:2 andM:0];
    [command lineFeed];
    [command addText:@"FLASH Bitmap serial number 3:\n"];
    [command printBitmapToFlashWithN:3 andM:0];
    [command printAndFeedLines:6];
    [command cutPaperWithMode:PaperCutModePartialCut];
    [self printWithData:[command getCommand]];
}

- (IBAction)downloadRamAction:(id)sender {
    UIImage *image = [UIImage imageNamed:@"nv_test"];
    XPOSCommand *command = [[XPOSCommand alloc] init];
    [command defineDownloadedRAMBmp:image bmpType:BMPDithering];
    [command printAndFeedLines:6];
    [command cutPaperWithMode:PaperCutModePartialCut];
    [self printWithData:[command getCommand]];
}


- (IBAction)printRamAction:(id)sender {
    XPOSCommand *command = [[XPOSCommand alloc] init];
    [command printDownloadRAMBmp:BmpNormal];
    [command printAndFeedLines:6];
    [command cutPaperWithMode:PaperCutModePartialCut];
    [self printWithData:[command getCommand]];
}

- (IBAction)openCashBoxAction:(id)sender {
    XPOSCommand *command = [[XPOSCommand alloc] init];
    [command initializePrinter];
    [command openCashBoxWithPinNumber:CashDrawerPin2];
    [self printWithData:[command getCommand]];
}

- (IBAction)checkCashBoxStatusAction:(id)sender {
    if (_connectType == ConnectTypeBluetooth) {
        [_bleManager cashBoxCheck:^(NSData *status) {
            [self cashBoxStatus:status];
        }];
    } else if (_connectType == ConnectTypeWiFi) {
        [_wifiManager cashBoxCheck:^(NSData *status) {
            [self cashBoxStatus:status];
        }];
    }
}

- (IBAction)printSampleAction:(id)sender {
    XPOSCommand *command = [[XPOSCommand alloc] init];
    [command initializePrinter];
    [command addText:@"              Las vegas,NV5208"];
    [command printAndFeedLines:2];
    [command selectAlignment:POSAlignmentLeft];
    [command addText:@"Ticket #30-57320                User:HAPPY"];
    [command lineFeed];
    [command addText:@"Station:52-102             Sales Rep HAPPY"];
    [command lineFeed];
    [command addText:@"10/10/2019 3:55:01PM"];
    [command lineFeed];
    [command addText:@"------------------------------------------"];
    [command lineFeed];
    [command addText:@"Item             QTY     Price       Total"];
    [command lineFeed];
    [command addText:@"Description"];
    [command lineFeed];
    [command addText:@"------------------------------------------"];
    [command lineFeed];
    [command addText:@"100328            1       7.99        7.99"];
    [command lineFeed];
    [command addText:@"MAGARITA MIX"];
    [command lineFeed];
    [command addText:@"680015            4       0.99        3.96"];
    [command lineFeed];
    [command addText:@"LIME"];
    [command lineFeed];
    [command addText:@"102501            1      43.99       43.99"];
    [command lineFeed];
    [command addText:@"VODKA"];
    [command lineFeed];
    [command addText:@"021048            1       4.99        4.99"];
    [command lineFeed];
    [command addText:@"ORANGE 320Z"];
    [command lineFeed];
    [command addText:@"------------------------------------------"];
    [command lineFeed];
    [command addText:@"Subtobal                             60.93"];
    [command lineFeed];
    [command addText:@"8.1% Sales Tax                        4.21"];
    [command lineFeed];
    [command addText:@"2% Concession Recov                   1.04"];
    [command lineFeed];
    [command addText:@"------------------------------------------"];
    [command lineFeed];
    [command addText:@"Total                                66.18"];
    [command printAndFeedLines:2];
    [command selectAlignment:POSAlignmentCenter];
    [command printQRCodeWithContent:@"www.google.com" moduleSize:6 ecLevel:QRErrorCorrectionLevelL];
    [command printAndFeedLines:5];
    [command cutPaper];
    [self printWithData:[command getCommand]];
}

- (IBAction)checkStatusAction:(id)sender {
    
    if (_connectType == ConnectTypeBluetooth) {
        [_bleManager printerPOSStatus:^(NSData *status) {
            [self posPrinterState:status];
        }];
    } else if (_connectType == ConnectTypeWiFi) {
        [_wifiManager printerPOSStatus:^(NSData *status) {
            [self posPrinterState:status];
        }];
    }
}

- (void)blePrintWithData:(NSData *)data {
    NSUInteger maxWriteLength = [[XBLEManager sharedInstance].writePeripheral maximumWriteValueLengthForType:CBCharacteristicWriteWithoutResponse];
    if (data.length > maxWriteLength) {
        NSLog(@"Send in packets");
        NSUInteger offset = 0;
        while (offset < data.length) {
            NSUInteger chunkLength = MIN(maxWriteLength, data.length - offset);
            NSData *chunk = [data subdataWithRange:NSMakeRange(offset, chunkLength)];
            [[XBLEManager sharedInstance] writeCommandWithData:chunk];
            offset += chunkLength;
        }
    } else {
        NSLog(@"Direct Send");
        [[XBLEManager sharedInstance] writeCommandWithData:data];
    }
}

- (void)posPrinterState:(NSData *)data {

    unsigned status = 0;
    if (data.length == 1) {
        const Byte *byte = (Byte *)[data bytes];
        status = byte[0];
    } else if (data.length == 2) {
        const Byte *byte = (Byte *)[data bytes];
        status = byte[1];
    }
    
    if (status == 0x12) {
        [self.view makeToast:@"Normal" duration:1.f position:CSToastPositionCenter];
    } else if (status == 0x16) {
        [self.view makeToast:@"Cover opened" duration:1.f position:CSToastPositionCenter];
    } else if (status == 0x32) {
        [self.view makeToast:@"Paper end" duration:1.f position:CSToastPositionCenter];
    } else if (status == 0x36) {
        [self.view makeToast:@"Cover opened & Paper end" duration:1.f position:CSToastPositionCenter];
    } else if (status == 0x1A) {
        [self.view makeToast:@"Feeding" duration:1.f position:CSToastPositionCenter];
    } else {
        [self.view makeToast:@"Other error" duration:1.f position:CSToastPositionCenter];
    }
}

- (void)cashBoxStatus:(NSData *)data {
    
    if (data.length == 0) return;
    
    if (data.length == 1) {
        const Byte *byte = (Byte *)[data bytes];
        unsigned status = byte[0];
        if (status == 0x00) {
            [self.view makeToast:@"Cash box open" duration:1.f position:CSToastPositionCenter];
        } else if (status == 0x01) {
            [self.view makeToast:@"Cash box closed" duration:1.f position:CSToastPositionCenter];
        }
    }
}

- (void)printWithData:(NSData *)printData {
    
    if (_connectType == ConnectTypeBluetooth) {
        [self blePrintWithData:printData];
    } else if (_connectType == ConnectTypeWiFi) {
        [_wifiManager writeCommandWithData:printData];
    } else {
        [self.view makeToast:@"printer no connect" duration:1.f position:CSToastPositionCenter];
    }
}

#pragma mark - XBLEManagerDelegate

- (void)xbleDisconnectPeripheral:(CBPeripheral *)peripheral error:(NSError *)error {
    [self.view makeToast:@"disconnect.." duration:1.f position:CSToastPositionCenter];
    [self.navigationController popViewControllerAnimated:YES];
}

#pragma mark - XWIFIManagerDelegate

- (void)xWifiDisconnectWithError:(NSError *)error {
    [self.view makeToast:@"disconnect.." duration:1.f position:CSToastPositionCenter];
    [self.navigationController popViewControllerAnimated:YES];
}

- (void)xWifiReceiveValueForData:(NSData *)data {
    NSLog(@"wifi receive value:%@",data);
}

@end
