//
//  RFIDCustomView.m
//  PrinterDemo
//
//  Created by Apple Mac mini intel on 2024/7/22.
//

#import "RFIDCustomView.h"

@implementation RFIDCustomView

@end

@interface TagCorrectView ()

@property (nonatomic, strong) UILabel *infoLabel;    // Label
@property (nonatomic, strong) UIButton *sendButton;   // Button

- (instancetype)initWithFrame:(CGRect)frame;

@end

@implementation TagCorrectView

- (instancetype)initWithFrame:(CGRect)frame {
    self = [super initWithFrame:frame];
    if (self) {
        [self setupView];
    }
    return self;
}

- (void)setupView {
    // Create label
    self.infoLabel = [[UILabel alloc] init];
    self.infoLabel.text = @"Correcting Response Position";
    self.infoLabel.font = [UIFont systemFontOfSize:16];
    
    // Create button
    self.sendButton = [UIButton buttonWithType:UIButtonTypeSystem];
    [self.sendButton setTitle:@"Send" forState:UIControlStateNormal];
    [self.sendButton addTarget:self action:@selector(sendTagCorrect) forControlEvents:UIControlEventTouchUpInside];
    
    // Layout
    UIStackView *stackView = [[UIStackView alloc] initWithArrangedSubviews:@[
        self.infoLabel,
        self.sendButton
    ]];
    stackView.axis = UILayoutConstraintAxisHorizontal; // Horizontal layout
    stackView.spacing = 10; // Spacing
    stackView.translatesAutoresizingMaskIntoConstraints = NO;
    
    [self addSubview:stackView];
    
    // Constraints
    [NSLayoutConstraint activateConstraints:@[
        [stackView.centerXAnchor constraintEqualToAnchor:self.centerXAnchor],
        [stackView.centerYAnchor constraintEqualToAnchor:self.centerYAnchor],
        [stackView.leadingAnchor constraintEqualToAnchor:self.leadingAnchor constant:20],
        [stackView.trailingAnchor constraintEqualToAnchor:self.trailingAnchor constant:-20]
    ]];
}

#pragma mark - Button Action

- (void)sendTagCorrect {
    // Call callback
    if (self.onSend) {
        self.onSend();
    }
}

@end

@interface TagCorrectionView ()

@property (nonatomic, strong) UITextField *startField;           // Start input field
@property (nonatomic, strong) UITextField *endField;             // End input field
@property (nonatomic, strong) UITextField *startPosField;        // Start position input field
@property (nonatomic, strong) UITextField *endPosField;          // End position input field
@property (nonatomic, strong) UISwitch *modeSwitch;              // Auto/manual switch
@property (nonatomic, strong) UIButton *submitButton;            // Submit button
@property (nonatomic, strong) UILabel *headLabel;                // Title label
@property (nonatomic, strong) UILabel *modeLabel;                // Mode label
@property (nonatomic, strong) UISegmentedControl *startPosSegment; // Start position segment selector
@property (nonatomic, strong) UISegmentedControl *endPosSegment;  // End position segment selector

- (instancetype)initWithFrame:(CGRect)frame;

@end

@implementation TagCorrectionView

- (instancetype)initWithFrame:(CGRect)frame {
    self = [super initWithFrame:frame];
    if (self) {
        [self setupView];
    }
    return self;
}

- (void)setupView {
    // Create text fields
    self.startField = [self createTextFieldWithPlaceholder:@"Start String"];
    self.endField = [self createTextFieldWithPlaceholder:@"End String"];
    
    self.startPosField = [self createTextFieldWithPlaceholder:@"Start Position"];
    self.startPosField.keyboardType = UIKeyboardTypeNumberPad;
    self.startPosSegment = [self createSegmentedControlWithItems:@[@"Forward", @"Backward"]];
    
    self.endPosField = [self createTextFieldWithPlaceholder:@"End Position"];
    self.endPosField.keyboardType = UIKeyboardTypeNumberPad;
    self.endPosSegment = [self createSegmentedControlWithItems:@[@"Forward", @"Backward"]];
    
    self.headLabel = [[UILabel alloc] init];
    self.headLabel.text = @"Correcting Response Position";
    self.startField.text = @"start";
    self.endField.text = @"end";

    // Create switch and label
    self.modeSwitch = [[UISwitch alloc] init];
    self.modeLabel = [[UILabel alloc] init];
    self.modeLabel.text = @"Antenna and Read/Write Power Level Detection";
    
    // Create submit button
    self.submitButton = [UIButton buttonWithType:UIButtonTypeSystem];
    [self.submitButton setTitle:@"Submit" forState:UIControlStateNormal];
    [self.submitButton addTarget:self action:@selector(submit) forControlEvents:UIControlEventTouchUpInside];
    
    // Layout
    UIStackView *stackView = [[UIStackView alloc] initWithArrangedSubviews:@[
        self.headLabel,
        self.startField,
        self.endField,
        [self createStackWithSegment:self.startPosSegment andField:self.startPosField],
        [self createStackWithSegment:self.endPosSegment andField:self.endPosField],
        [self createStackWithLabel:self.modeLabel andSwitch:self.modeSwitch],
        self.submitButton
    ]];
    stackView.axis = UILayoutConstraintAxisVertical;
    stackView.spacing = 10;
    stackView.translatesAutoresizingMaskIntoConstraints = NO;
    
    [self addSubview:stackView];
    
    // Constraints
    [NSLayoutConstraint activateConstraints:@[
        [stackView.centerXAnchor constraintEqualToAnchor:self.centerXAnchor],
        [stackView.centerYAnchor constraintEqualToAnchor:self.centerYAnchor],
        [stackView.leadingAnchor constraintEqualToAnchor:self.leadingAnchor constant:20],
        [stackView.trailingAnchor constraintEqualToAnchor:self.trailingAnchor constant:-20]
    ]];
}

- (UITextField *)createTextFieldWithPlaceholder:(NSString *)placeholder {
    UITextField *textField = [[UITextField alloc] init];
    textField.borderStyle = UITextBorderStyleRoundedRect;
    textField.placeholder = placeholder;
    return textField;
}

- (UISegmentedControl *)createSegmentedControlWithItems:(NSArray *)items {
    UISegmentedControl *segmentedControl = [[UISegmentedControl alloc] initWithItems:items];
    segmentedControl.selectedSegmentIndex = 0;
    return segmentedControl;
}

- (UIStackView *)createStackWithSegment:(UISegmentedControl *)segment andField:(UITextField *)field {
    UIStackView *stack = [[UIStackView alloc] initWithArrangedSubviews:@[segment, field]];
    stack.axis = UILayoutConstraintAxisHorizontal;
    stack.spacing = 10;
    return stack;
}

- (UIStackView *)createStackWithLabel:(UILabel *)label andSwitch:(UISwitch *)switchControl {
    UIStackView *stack = [[UIStackView alloc] initWithArrangedSubviews:@[label, switchControl]];
    stack.axis = UILayoutConstraintAxisHorizontal;
    stack.spacing = 10;
    return stack;
}

- (void)submit {
    NSString *a = self.startField.text.length > 0 ? self.startField.text : @"start";
    NSString *b = self.endField.text.length > 0 ? self.endField.text : @"end";
    NSString *c = self.startPosField.text.length > 0 ? self.startPosField.text : @"0";
    NSString *d = self.endPosField.text.length > 0 ? self.endPosField.text : @"0";
    NSString *e = self.modeSwitch.isOn ? @"M" : @"A"; // Auto/manual
    
    NSString *startPos = [NSString stringWithFormat:@"%@%@",(self.startPosSegment.selectedSegmentIndex == 0)?@"F":@"B",c];
    NSString *endPos = [NSString stringWithFormat:@"%@%@",(self.endPosSegment.selectedSegmentIndex == 0)?@"F":@"B",d];
    
    // Call callback
    if (self.onSubmit) {
        self.onSubmit(a, b, startPos, endPos, e);
    }
}

@end


@interface RFIDPowerView ()<UIPickerViewDelegate,UIPickerViewDataSource>

@property (nonatomic, strong) UITextField *readPowerField;      // Read power input field
@property (nonatomic, strong) UITextField *writePowerField;     // Write power input field
@property (nonatomic, strong) UIButton *submitButton;            // Submit button
//@property (nonatomic, strong) UIPickerView *pickerView;         // Picker view
@property (nonatomic, strong) NSArray *powerOptions;             // Picker options
@property (nonatomic, strong) UILabel *headLabel;                // Title label
//@property (nonatomic, strong) UILabel *antennaLabel;            // Antenna options label

- (instancetype)initWithFrame:(CGRect)frame;

@end

@implementation RFIDPowerView

- (instancetype)initWithFrame:(CGRect)frame {
    self = [super initWithFrame:frame];
    if (self) {
        [self setupView];
    }
    return self;
}

- (void)setupView {
    // Initialize picker options
    self.powerOptions = @[@"A1", @"A2", @"A3", @"A4",
                          @"B1", @"B2", @"B3", @"B4",
                          @"C1", @"C2", @"C3", @"C4",
                          @"D1", @"D2", @"D3", @"D4",
                          @"E1", @"E2", @"E3", @"E4"];
    
    // Create read power input field
    self.readPowerField = [self createTextFieldWithPlaceholder:@"Read Power(value: 0 to 30)"];
    self.readPowerField.keyboardType = UIKeyboardTypeNumberPad; // Numeric keyboard
    
    // Create write power input field
    self.writePowerField = [self createTextFieldWithPlaceholder:@"Write Power(value: 0 to 30)"];
    self.writePowerField.keyboardType = UIKeyboardTypeNumberPad; // Numeric keyboard
    
    [self.readPowerField addTarget:self action:@selector(textFieldDidChange:) forControlEvents:UIControlEventEditingChanged];
    
    [self.writePowerField addTarget:self action:@selector(textFieldDidChange:) forControlEvents:UIControlEventEditingChanged];
    
    // Create picker view
//    self.pickerView = [[UIPickerView alloc] init];
//    self.pickerView.delegate = self;
//    self.pickerView.dataSource = self;
    
    self.headLabel = [[UILabel alloc] init];
    self.headLabel.text = @"Set RFID Read and Write Power Levels";
    
//    self.antennaLabel = [[UILabel alloc] init];
//    self.antennaLabel.text = @"Antenna Selection";
    
    // Create submit button
    self.submitButton = [UIButton buttonWithType:UIButtonTypeSystem];
    [self.submitButton setTitle:@"Submit" forState:UIControlStateNormal];
    [self.submitButton addTarget:self action:@selector(submit) forControlEvents:UIControlEventTouchUpInside];
    
    // Layout
    UIStackView *stackView = [[UIStackView alloc] initWithArrangedSubviews:@[
        self.headLabel,
        self.readPowerField,
        self.writePowerField,
//        self.antennaLabel,
//        self.pickerView,
        self.submitButton
    ]];
    stackView.axis = UILayoutConstraintAxisVertical;
    stackView.spacing = 10;
    stackView.translatesAutoresizingMaskIntoConstraints = NO;
    
    [self addSubview:stackView];
    
    // Constraints
    [NSLayoutConstraint activateConstraints:@[
        [stackView.centerXAnchor constraintEqualToAnchor:self.centerXAnchor],
        [stackView.centerYAnchor constraintEqualToAnchor:self.centerYAnchor],
        [stackView.leadingAnchor constraintEqualToAnchor:self.leadingAnchor constant:20],
        [stackView.trailingAnchor constraintEqualToAnchor:self.trailingAnchor constant:-20]
    ]];
    
}

- (UITextField *)createTextFieldWithPlaceholder:(NSString *)placeholder {
    UITextField *textField = [[UITextField alloc] init];
    textField.borderStyle = UITextBorderStyleRoundedRect;
    textField.placeholder = placeholder;
    return textField;
}

- (void)submit {
    NSString *readPower = self.readPowerField.text.length > 0 ? self.readPowerField.text : @"0";
    NSString *writePower = self.writePowerField.text.length > 0 ? self.writePowerField.text : @"0";
    
    // Call callback
    if (self.onSubmit) {
        self.onSubmit(readPower, writePower);
    }
}

- (void)textFieldDidChange:(UITextField *)textField {
    NSString *numberRegex = @"^[0-9]{0,2}$";
    NSPredicate *predicate = [NSPredicate predicateWithFormat:@"SELF MATCHES %@", numberRegex];
    
    if (![predicate evaluateWithObject:textField.text]) {
        textField.text = [textField.text substringToIndex:textField.text.length - 1];
    }
}

// UIPickerViewDelegate and UIPickerViewDataSource methods
- (NSInteger)numberOfComponentsInPickerView:(UIPickerView *)pickerView {
    return 1;
}

- (NSInteger)pickerView:(UIPickerView *)pickerView numberOfRowsInComponent:(NSInteger)component {
    return self.powerOptions.count;
}

- (NSString *)pickerView:(UIPickerView *)pickerView titleForRow:(NSInteger)row forComponent:(NSInteger)component {
    return self.powerOptions[row];
}

@end

@implementation BottomView

- (instancetype)initWithFrame:(CGRect)frame {
    self = [super initWithFrame:frame];
    if (self) {
        [self setupView];
    }
    return self;
}

- (void)setupView {
    // Label
    self.contentLabel = [[UILabel alloc] init];
    self.contentLabel.text = @"Read Content"; // Changed to English
    self.contentLabel.translatesAutoresizingMaskIntoConstraints = NO;
    [self addSubview:self.contentLabel];

    // Button
    self.clearButton = [UIButton buttonWithType:UIButtonTypeSystem];
    [self.clearButton setTitle:@"Clear" forState:UIControlStateNormal];
    [self.clearButton addTarget:self action:@selector(clearContent) forControlEvents:UIControlEventTouchUpInside];
    self.clearButton.translatesAutoresizingMaskIntoConstraints = NO;
    [self addSubview:self.clearButton];

    // TextView
    self.textView = [[UITextView alloc] init];
    self.textView.editable = NO; // Set to non-editable
    self.textView.layer.cornerRadius = 8; // Set corner radius
    self.textView.layer.borderWidth = 1; // Set border width
    self.textView.layer.borderColor = [[UIColor lightGrayColor] CGColor]; // Set border color
    self.textView.textContainerInset = UIEdgeInsetsMake(8, 8, 8, 8); // Set padding
    self.textView.translatesAutoresizingMaskIntoConstraints = NO;
    [self addSubview:self.textView];

    // Layout
    [NSLayoutConstraint activateConstraints:@[
        [self.contentLabel.topAnchor constraintEqualToAnchor:self.topAnchor constant:0],
        [self.contentLabel.leadingAnchor constraintEqualToAnchor:self.leadingAnchor constant:16],
        
        [self.clearButton.trailingAnchor constraintEqualToAnchor:self.trailingAnchor constant:-16],

        [self.textView.topAnchor constraintEqualToAnchor:self.contentLabel.bottomAnchor constant:8],
        [self.textView.leadingAnchor constraintEqualToAnchor:self.leadingAnchor constant:16], // Add left margin
        [self.textView.trailingAnchor constraintEqualToAnchor:self.trailingAnchor constant:-16], // Add right margin
        [self.textView.bottomAnchor constraintEqualToAnchor:self.bottomAnchor constant:-8], // Add bottom margin
    ]];
}

- (void)clearContent {
    self.textView.text = @""; // Clear the content of the TextView
}

@end


@interface RFIDTagPasswordView ()

@property (nonatomic, strong) UILabel *accessPasswordLabel;
@property (nonatomic, strong) UILabel *killPasswordLabel;
//@property (nonatomic, strong) UILabel *typeLabel;
@property (nonatomic, strong) UILabel *accessStateLabel;
@property (nonatomic, strong) UILabel *killStateLabel;

@property (nonatomic, strong) UITextField *accessPasswordField;        // Password input field
@property (nonatomic, strong) UITextField *killPasswordField;        // Password input field
//@property (nonatomic, strong) UISegmentedControl *passwordTypeSeg; // Tag password type
@property (nonatomic, strong) UISegmentedControl *accessPasswordStateSeg; // Tag password state
@property (nonatomic, strong) UISegmentedControl *killPasswordStateSeg; // Tag password type
@property (nonatomic, strong) UIButton *submitButton;            // Submit button

- (instancetype)initWithFrame:(CGRect)frame;

@end

@implementation RFIDTagPasswordView

- (instancetype)initWithFrame:(CGRect)frame {
    self = [super initWithFrame:frame];
    if (self) {
        [self setupView];
    }
    return self;
}

- (void)setupView {
    
    self.accessPasswordLabel = [[UILabel alloc] init];
    self.accessPasswordLabel.text = @"Enter Access Password"; // Changed to English
    self.accessPasswordLabel.font = [UIFont systemFontOfSize:16];
    
    self.killPasswordLabel = [[UILabel alloc] init];
    self.killPasswordLabel.text = @"Enter Kill Password"; // Changed to English
    self.killPasswordLabel.font = [UIFont systemFontOfSize:16];
    
    self.killStateLabel = [[UILabel alloc] init];
    self.killStateLabel.text = @"Kill Password State"; // Changed to English
    self.killStateLabel.font = [UIFont systemFontOfSize:16];
    
    self.accessStateLabel = [[UILabel alloc] init];
    self.accessStateLabel.text = @"Access Password State"; // Changed to English
    self.accessStateLabel.font = [UIFont systemFontOfSize:16];
    
    // Create password input field
    self.accessPasswordField = [[UITextField alloc] init];
    self.accessPasswordField.borderStyle = UITextBorderStyleRoundedRect;
    self.accessPasswordField.placeholder = @"Please enter an 8-digit hexadecimal password"; // Changed to English
    self.accessPasswordField.keyboardType = UIKeyboardTypeASCIICapable; // ASCII keyboard
    [self.accessPasswordField addTarget:self action:@selector(textFieldDidChange:) forControlEvents:UIControlEventEditingChanged];
    
    
    self.killPasswordField = [[UITextField alloc] init];
    self.killPasswordField.borderStyle = UITextBorderStyleRoundedRect;
    self.killPasswordField.placeholder = @"Please enter an 8-digit hexadecimal password"; // Changed to English
    self.killPasswordField.keyboardType = UIKeyboardTypeASCIICapable; // ASCII keyboard
    [self.killPasswordField addTarget:self action:@selector(textFieldDidChange:) forControlEvents:UIControlEventEditingChanged];
    
    // Create segmented control for password state
    self.accessPasswordStateSeg = [[UISegmentedControl alloc] initWithItems:@[@"Unlock", @"Lock", @"Open", @"Protect"]];
    self.accessPasswordStateSeg.selectedSegmentIndex = 0;
    
    self.killPasswordStateSeg = [[UISegmentedControl alloc] initWithItems:@[@"Unlock", @"Lock", @"Open", @"Protect"]];
    self.killPasswordStateSeg.selectedSegmentIndex = 0;
    
    // Create submit button
    self.submitButton = [UIButton buttonWithType:UIButtonTypeSystem];
    [self.submitButton setTitle:@"Submit" forState:UIControlStateNormal]; // Changed to English
    [self.submitButton addTarget:self action:@selector(submit) forControlEvents:UIControlEventTouchUpInside];
    
    // Layout
    UIStackView *stackView = [[UIStackView alloc] initWithArrangedSubviews:@[
        self.accessPasswordLabel,
        self.accessPasswordField,
        self.accessStateLabel,
        self.accessPasswordStateSeg,
        
        self.killPasswordLabel,
        self.killPasswordField,
        self.killStateLabel,
        self.killPasswordStateSeg,
        
        self.submitButton
    ]];
    stackView.axis = UILayoutConstraintAxisVertical;
    stackView.spacing = 10;
    stackView.translatesAutoresizingMaskIntoConstraints = NO;
    
    [self addSubview:stackView];
    
    // Constraints
    [NSLayoutConstraint activateConstraints:@[
        [stackView.centerXAnchor constraintEqualToAnchor:self.centerXAnchor],
        [stackView.centerYAnchor constraintEqualToAnchor:self.centerYAnchor],
        [stackView.leadingAnchor constraintEqualToAnchor:self.leadingAnchor constant:20],
        [stackView.trailingAnchor constraintEqualToAnchor:self.trailingAnchor constant:-20]
    ]];
}

- (void)textFieldDidChange:(UITextField *)textField {
    // Limit input to 8 hexadecimal characters
    NSString *hexRegex = @"^[0-9A-Fa-f]{0,8}$";
    NSPredicate *predicate = [NSPredicate predicateWithFormat:@"SELF MATCHES %@", hexRegex];
    
    if (![predicate evaluateWithObject:textField.text]) {
        // If input is invalid, remove the last character
        textField.text = [textField.text substringToIndex:textField.text.length - 1];
    }
}

- (NSString *)returnPasswordState:(NSInteger)index {
    switch (index) {
        case 0: return @"U";
        case 1: return @"L";
        case 2: return @"O";
        case 3: return @"P";
        default: return @"";
    }
}

- (void)submit {
    // Call callback
    if (self.onSubmit) {
        self.onSubmit(self.accessPasswordField.text, [self returnPasswordState:self.accessPasswordStateSeg.selectedSegmentIndex], self.killPasswordField.text, [self returnPasswordState:self.killPasswordStateSeg.selectedSegmentIndex]);
    }
}

@end



@interface ResetCounterView() <UIPickerViewDelegate, UIPickerViewDataSource>

@property (nonatomic, strong) UILabel *titleLabel;          // Title label
@property (nonatomic, strong) UIPickerView *pickerView;     // Picker
@property (nonatomic, strong) UIButton *resetButton;         // Reset button

- (instancetype)initWithFrame:(CGRect)frame;

@end

@implementation ResetCounterView

- (instancetype)initWithFrame:(CGRect)frame {
    self = [super initWithFrame:frame];
    if (self) {
        [self setupView];
    }
    return self;
}

- (void)setupView {
    // Create title label
    self.titleLabel = [[UILabel alloc] init];
    self.titleLabel.text = @"Reset Advanced Counter"; // Changed to English
    
    // Create picker
    self.pickerView = [[UIPickerView alloc] init];
    self.pickerView.delegate = self;
    self.pickerView.dataSource = self;
    
    // Create reset button
    self.resetButton = [UIButton buttonWithType:UIButtonTypeSystem];
    [self.resetButton setTitle:@"Reset" forState:UIControlStateNormal]; // Changed to English
    [self.resetButton addTarget:self action:@selector(resetCounter) forControlEvents:UIControlEventTouchUpInside];
    
    // Layout
    UIStackView *stackView = [[UIStackView alloc] initWithArrangedSubviews:@[
        self.titleLabel,
        self.pickerView,
        self.resetButton
    ]];
    stackView.axis = UILayoutConstraintAxisVertical;
    stackView.spacing = 20;
    stackView.translatesAutoresizingMaskIntoConstraints = NO;
    
    [self addSubview:stackView];
    
    // Constraints
    [NSLayoutConstraint activateConstraints:@[
        [stackView.centerXAnchor constraintEqualToAnchor:self.centerXAnchor],
        [stackView.centerYAnchor constraintEqualToAnchor:self.centerYAnchor],
        [stackView.leadingAnchor constraintEqualToAnchor:self.leadingAnchor constant:20],
        [stackView.trailingAnchor constraintEqualToAnchor:self.trailingAnchor constant:-20]
    ]];
}

#pragma mark - UIPickerView DataSource & Delegate

- (NSInteger)numberOfComponentsInPickerView:(UIPickerView *)pickerView {
    return 1; // Only one column
}

- (NSInteger)pickerView:(UIPickerView *)pickerView numberOfRowsInComponent:(NSInteger)component {
    return 6; // Six options
}

- (NSString *)pickerView:(UIPickerView *)pickerView titleForRow:(NSInteger)row forComponent:(NSInteger)component {
    switch (row) {
        case 0: return @"Reset Counter 1"; // Changed to English
        case 1: return @"Reset Counter 2"; // Changed to English
        case 2: return @"Reset Valid RFID Tag Counter"; // Changed to English
        case 3: return @"Reset Invalid RFID Tag Counter"; // Changed to English
        case 4: return @"Reset Head Cleaning Counter"; // Changed to English
        case 5: return @"Reset Head Replacement Counter and Head Cleaning Counter"; // Changed to English
        default: return @"";
    }
}

#pragma mark - Reset Action

- (NSString *)returnCounter:(NSInteger)index {
    switch (index) {
        case 0: return @"1";
        case 1: return @"2";
        case 2: return @"3";
        case 3: return @"4";
        case 4: return @"C";
        case 5: return @"R";
        default: return @"";
    }
}

- (void)resetCounter {
    NSInteger selectedRow = [self.pickerView selectedRowInComponent:0];
    
    // Call callback
    if (self.onReset) {
        self.onReset([self returnCounter:selectedRow]);
    }
}

@end



@interface RFIDDataLogView ()

@property (nonatomic, strong) UILabel *infoLabel;    // Label
@property (nonatomic, strong) UIButton *sendButton;   // Button

- (instancetype)initWithFrame:(CGRect)frame;

@end

@implementation RFIDDataLogView

- (instancetype)initWithFrame:(CGRect)frame {
    self = [super initWithFrame:frame];
    if (self) {
        [self setupView];
    }
    return self;
}

- (void)setupView {
    // Create label
    self.infoLabel = [[UILabel alloc] init];
    self.infoLabel.text = @"Return RFID data log to host";
    self.infoLabel.font = [UIFont systemFontOfSize:16];
    
    // Create button
    self.sendButton = [UIButton buttonWithType:UIButtonTypeSystem];
    [self.sendButton setTitle:@"Send" forState:UIControlStateNormal];
    [self.sendButton addTarget:self action:@selector(sendDataLog) forControlEvents:UIControlEventTouchUpInside];
    
    // Layout
    UIStackView *stackView = [[UIStackView alloc] initWithArrangedSubviews:@[
        self.infoLabel,
        self.sendButton
    ]];
    stackView.axis = UILayoutConstraintAxisHorizontal; // Horizontal layout
    stackView.spacing = 10; // Spacing
    stackView.translatesAutoresizingMaskIntoConstraints = NO;
    
    [self addSubview:stackView];
    
    // Constraints
    [NSLayoutConstraint activateConstraints:@[
        [stackView.centerXAnchor constraintEqualToAnchor:self.centerXAnchor],
        [stackView.centerYAnchor constraintEqualToAnchor:self.centerYAnchor],
        [stackView.leadingAnchor constraintEqualToAnchor:self.leadingAnchor constant:20],
        [stackView.trailingAnchor constraintEqualToAnchor:self.trailingAnchor constant:-20]
    ]];
}

#pragma mark - Button Action

- (void)sendDataLog {
    // Call callback
    if (self.onSend) {
        self.onSend();
    }
}

@end



@interface RFIDReaderView ()

@property (nonatomic, strong) UILabel *infoLabel;    // Label
@property (nonatomic, strong) UIButton *sendButton;   // Button
@property (nonatomic, strong) UIView *separatorLine; // Separator line

- (instancetype)initWithFrame:(CGRect)frame;

@end

@implementation RFIDReaderView

- (instancetype)initWithFrame:(CGRect)frame {
    self = [super initWithFrame:frame];
    if (self) {
        [self setupView];
    }
    return self;
}

- (void)setupView {
    // Create label
    self.infoLabel = [[UILabel alloc] init];
    self.infoLabel.text = @"Read unique RFID chip serial number";
    self.infoLabel.font = [UIFont systemFontOfSize:16];
    
    // Create button
    self.sendButton = [UIButton buttonWithType:UIButtonTypeSystem];
    [self.sendButton setTitle:@"Send" forState:UIControlStateNormal];
    [self.sendButton addTarget:self action:@selector(sendData) forControlEvents:UIControlEventTouchUpInside];
    
    // Layout
    UIStackView *stackView = [[UIStackView alloc] initWithArrangedSubviews:@[
        self.infoLabel,
        self.sendButton
    ]];
    stackView.axis = UILayoutConstraintAxisHorizontal; // Horizontal layout
    stackView.spacing = 10; // Spacing
    stackView.translatesAutoresizingMaskIntoConstraints = NO;
    
    [self addSubview:stackView];
    
    // Create separator line
    self.separatorLine = [[UIView alloc] init];
    self.separatorLine.backgroundColor = [UIColor grayColor]; // Set separator line color
    self.separatorLine.translatesAutoresizingMaskIntoConstraints = NO;
    [self addSubview:self.separatorLine];
    
    // Constraints
    [NSLayoutConstraint activateConstraints:@[
        [stackView.centerXAnchor constraintEqualToAnchor:self.centerXAnchor],
        [stackView.centerYAnchor constraintEqualToAnchor:self.centerYAnchor],
        [stackView.leadingAnchor constraintEqualToAnchor:self.leadingAnchor constant:20],
        [stackView.trailingAnchor constraintEqualToAnchor:self.trailingAnchor constant:-20],
        
        // Separator line constraints
        [self.separatorLine.leadingAnchor constraintEqualToAnchor:self.leadingAnchor constant:10],
        [self.separatorLine.trailingAnchor constraintEqualToAnchor:self.trailingAnchor constant:-10],
        [self.separatorLine.topAnchor constraintEqualToAnchor:stackView.bottomAnchor constant:20],
        [self.separatorLine.heightAnchor constraintEqualToConstant:1], // Set separator line height
        [self.separatorLine.bottomAnchor constraintEqualToAnchor:self.bottomAnchor constant:-10]
    ]];
}

#pragma mark - Button Action

- (void)sendData {
    // Call callback
    if (self.onSend) {
        self.onSend();
    }
}

@end

@interface RFIDTagReaderView () <UIPickerViewDelegate, UIPickerViewDataSource>
@property (nonatomic, strong) UILabel *infoLabel;            // Info label
@property (nonatomic, strong) UILabel *retryLabel;            // Retry count label
@property (nonatomic, strong) UIPickerView *retryPicker;      // Retry count picker
@property (nonatomic, strong) UILabel *operationLabel;        // Post-read operation label
@property (nonatomic, strong) UISwitch *writeSwitch;          // Write after feed switch
@property (nonatomic, strong) UIButton *submitButton;          // Submit button
@property (nonatomic, strong) UIView *separatorLine;           // Separator line

- (instancetype)initWithFrame:(CGRect)frame;

@end

@implementation RFIDTagReaderView

- (instancetype)initWithFrame:(CGRect)frame {
    self = [super initWithFrame:frame];
    if (self) {
        [self setupView];
    }
    return self;
}

- (void)setupView {
    
    // Create label
    self.infoLabel = [[UILabel alloc] init];
    self.infoLabel.text = @"Get RFID tag ID";
    self.infoLabel.font = [UIFont systemFontOfSize:16];
    
    // Create retry count label
    self.retryLabel = [[UILabel alloc] init];
    self.retryLabel.text = @"Retry Count";
    self.retryLabel.font = [UIFont systemFontOfSize:16];
    
    // Create retry count picker
    self.retryPicker = [[UIPickerView alloc] init];
    self.retryPicker.delegate = self;
    self.retryPicker.dataSource = self;
    
    // Create post-read operation label
    self.operationLabel = [[UILabel alloc] init];
    self.operationLabel.text = @"Feed paper after writing?";
    self.operationLabel.font = [UIFont systemFontOfSize:16];
    
    // Create write after feed switch
    self.writeSwitch = [[UISwitch alloc] init];
    
    // Create submit button
    self.submitButton = [UIButton buttonWithType:UIButtonTypeSystem];
    [self.submitButton setTitle:@"Submit" forState:UIControlStateNormal];
    [self.submitButton addTarget:self action:@selector(submitData) forControlEvents:UIControlEventTouchUpInside];

    // Create horizontal stack for operation label and switch
    UIStackView *operationStackView = [[UIStackView alloc] initWithArrangedSubviews:@[
        self.operationLabel,
        self.writeSwitch
    ]];
    operationStackView.axis = UILayoutConstraintAxisHorizontal; // Horizontal layout
    operationStackView.spacing = 20; // Set spacing to 20
    
    // Layout
    UIStackView *stackView = [[UIStackView alloc] initWithArrangedSubviews:@[
        self.infoLabel,
        self.retryLabel,
        self.retryPicker,
        operationStackView, // Use horizontal stack
        self.submitButton
    ]];
    stackView.axis = UILayoutConstraintAxisVertical; // Vertical layout
    stackView.spacing = 10; // Vertical spacing
    stackView.translatesAutoresizingMaskIntoConstraints = NO;
    
    [self addSubview:stackView];
    
    // Create separator line
    self.separatorLine = [[UIView alloc] init];
    self.separatorLine.backgroundColor = [UIColor grayColor]; // Set separator line color
    self.separatorLine.translatesAutoresizingMaskIntoConstraints = NO;
    [self addSubview:self.separatorLine];
    
    // Constraints
    [NSLayoutConstraint activateConstraints:@[
        [stackView.leadingAnchor constraintEqualToAnchor:self.leadingAnchor constant:20],
        [stackView.trailingAnchor constraintEqualToAnchor:self.trailingAnchor constant:-20],
        [stackView.topAnchor constraintEqualToAnchor:self.topAnchor constant:20],
        
        // Separator line constraints
        [self.separatorLine.leadingAnchor constraintEqualToAnchor:self.leadingAnchor constant:10],
        [self.separatorLine.trailingAnchor constraintEqualToAnchor:self.trailingAnchor constant:-10],
        [self.separatorLine.topAnchor constraintEqualToAnchor:stackView.bottomAnchor constant:20],
        [self.separatorLine.heightAnchor constraintEqualToConstant:1], // Set separator line height
        [self.separatorLine.bottomAnchor constraintEqualToAnchor:self.bottomAnchor constant:-10]
    ]];
}

#pragma mark - UIPickerView DataSource & Delegate

- (NSInteger)numberOfComponentsInPickerView:(UIPickerView *)pickerView {
    return 1; // Only one column
}

- (NSInteger)pickerView:(UIPickerView *)pickerView numberOfRowsInComponent:(NSInteger)component {
    return 11; // 0-10, a total of 11 options
}

- (NSString *)pickerView:(UIPickerView *)pickerView titleForRow:(NSInteger)row forComponent:(NSInteger)component {
    return [NSString stringWithFormat:@"%ld", (long)row]; // Return corresponding number
}

#pragma mark - Submit Data

- (void)submitData {
    NSInteger selectedRow = [self.retryPicker selectedRowInComponent:0];
    BOOL writeAfter = self.writeSwitch.isOn;
    
    // Call callback
    if (self.onSubmit) {
        self.onSubmit(selectedRow, writeAfter);
    }
}

@end



@interface RFIDWriteView() <UIPickerViewDelegate, UITextFieldDelegate>

@property (nonatomic, strong) UILabel *formatLabel;       // Format label
@property (nonatomic, strong) UILabel *contentLabel;      // Content label
//@property (nonatomic, strong) UILabel *blockNumberLabel; // Block number label
@property (nonatomic, strong) UILabel *memoryLabel;       // Memory label

@property (nonatomic, strong) UISegmentedControl *formatSegmentedControl; // Format selection
@property (nonatomic, strong) UITextField *contentTextField;            // Content input field
@property (nonatomic, strong) UISegmentedControl *memorySegmentedControl; // Memory segment selection
@property (nonatomic, strong) UIButton *submitButton;                   // Submit button


@property (nonatomic, strong) UILabel *readLabel;
//@property (nonatomic, strong) UILabel *printLabel;
@property (nonatomic, strong) UISwitch *readSwitch;
//@property (nonatomic, strong) UISwitch *printSwitch;

- (instancetype)initWithFrame:(CGRect)frame;

@end

@implementation RFIDWriteView

- (instancetype)initWithFrame:(CGRect)frame {
    self = [super initWithFrame:frame];
    if (self) {
        [self setupView];
    }
    return self;
}

- (void)setupView {
    
    self.formatLabel = [[UILabel alloc] init];
    self.formatLabel.text = @"Select the format of the content to write";
    self.formatLabel.font = [UIFont systemFontOfSize:16];
    
    self.contentLabel = [[UILabel alloc] init];
    self.contentLabel.text = @"Please enter content";
    self.contentLabel.font = [UIFont systemFontOfSize:16];
    
    self.memoryLabel = [[UILabel alloc] init];
    self.memoryLabel.text = @"Select memory segment, default is EPC96";
    self.memoryLabel.font = [UIFont systemFontOfSize:16];
    
    // Format selection Segmented Control
    self.formatSegmentedControl = [[UISegmentedControl alloc] initWithItems:@[@"ASCII", @"Hex"]];
    [self.formatSegmentedControl addTarget:self action:@selector(formatChanged) forControlEvents:UIControlEventValueChanged];
    self.formatSegmentedControl.selectedSegmentIndex = 1;
    
    // Content input field
    self.contentTextField = [[UITextField alloc] init];
    self.contentTextField.borderStyle = UITextBorderStyleRoundedRect;
    self.contentTextField.placeholder = @"Enter content";
    self.contentTextField.delegate = self;
    self.contentTextField.keyboardType = UIKeyboardTypeDefault;
    self.contentTextField.placeholder = @"Enter hexadecimal characters";
    
    self.readSwitch = [[UISwitch alloc] init];
    self.readLabel = [[UILabel alloc] init];
    self.readLabel.text = @"Read label content simultaneously";

//    self.printSwitch = [[UISwitch alloc] init];
//    self.printLabel = [[UILabel alloc] init];
//    self.printLabel.text = @"Print label content simultaneously";

    // Memory segment selection Segmented Control
    self.memorySegmentedControl = [[UISegmentedControl alloc] initWithItems:@[@"EPC96", @"EPC", @"User"]];
    self.memorySegmentedControl.selectedSegmentIndex = 0;
    
    // Submit button
    self.submitButton = [UIButton buttonWithType:UIButtonTypeSystem];
    [self.submitButton setTitle:@"Submit" forState:UIControlStateNormal];
    [self.submitButton addTarget:self action:@selector(submitData) forControlEvents:UIControlEventTouchUpInside];

    // Layout
    UIStackView *stackView = [[UIStackView alloc] initWithArrangedSubviews:@[
        self.formatLabel,
        self.formatSegmentedControl,
        self.contentLabel,
        self.contentTextField,
        self.memoryLabel,
        self.memorySegmentedControl,
        [self createStackWithLabel:self.readLabel andButton:self.readSwitch],
//        [self createStackWithLabel:self.printLabel andButton:self.printSwitch],
        self.submitButton
    ]];
    stackView.axis = UILayoutConstraintAxisVertical;
    stackView.spacing = 10;
    stackView.translatesAutoresizingMaskIntoConstraints = NO;

    [self addSubview:stackView];

    // Constraints
    [NSLayoutConstraint activateConstraints:@[
        [stackView.leadingAnchor constraintEqualToAnchor:self.leadingAnchor constant:20],
        [stackView.trailingAnchor constraintEqualToAnchor:self.trailingAnchor constant:-20],
        [stackView.topAnchor constraintEqualToAnchor:self.topAnchor constant:20],
        [stackView.bottomAnchor constraintEqualToAnchor:self.bottomAnchor constant:-20]
    ]];
}

- (UIStackView *)createStackWithLabel:(UILabel *)label andButton:(UISwitch *)switchControl {
    UIStackView *stack = [[UIStackView alloc] initWithArrangedSubviews:@[label, switchControl]];
    stack.axis = UILayoutConstraintAxisHorizontal;
    stack.spacing = 10;
    return stack;
}

- (void)formatChanged {
    // Limit input based on selected format
    NSString *selectedFormat = [self.formatSegmentedControl titleForSegmentAtIndex:self.formatSegmentedControl.selectedSegmentIndex];
    if ([selectedFormat isEqualToString:@"ASCII"]) {
        self.contentTextField.keyboardType = UIKeyboardTypeDefault;
        self.contentTextField.placeholder = @"Enter ASCII characters";
    } else if ([selectedFormat isEqualToString:@"Hex"]) {
        self.contentTextField.keyboardType = UIKeyboardTypeDefault;
        self.contentTextField.placeholder = @"Enter hexadecimal characters";
    } else if ([selectedFormat isEqualToString:@"EPC"]) {
        self.contentTextField.keyboardType = UIKeyboardTypeDefault;
        self.contentTextField.placeholder = @"Enter EPC characters";
    }
}

- (NSString *)returnFormat:(NSInteger)index {
    switch (index) {
        case 0: return @"A";
        case 1: return @"H";
        case 2: return @"E";
        default: return @"H";
    }
}

- (NSString *)returnMemory:(NSInteger)index {
    switch (index) {
        case 0: return @"E";
        case 1: return @"1";
        case 2: return @"3";
        default: return @"E";
    }
}

- (void)submitData {
    NSString *content = self.contentTextField.text;
    NSString *blockNumber = @"2";
    if (self.onSubmit) {
        self.onSubmit(content, blockNumber, [self returnMemory:self.memorySegmentedControl.selectedSegmentIndex], [self returnFormat:self.formatSegmentedControl.selectedSegmentIndex],self.readSwitch.on,NO);
    }
}

#pragma mark - UITextFieldDelegate

- (BOOL)textField:(UITextField *)textField shouldChangeCharactersInRange:(NSRange)range replacementString:(NSString *)string {
    // Limit input content
    if (textField == self.contentTextField) {
        NSString *selectedFormat = [self.formatSegmentedControl titleForSegmentAtIndex:self.formatSegmentedControl.selectedSegmentIndex];
        if ([selectedFormat isEqualToString:@"ASCII"]) {
            NSCharacterSet *allowedCharacters = [[NSCharacterSet alphanumericCharacterSet] invertedSet];
            return ([string rangeOfCharacterFromSet:allowedCharacters].location == NSNotFound);
        } else if ([selectedFormat isEqualToString:@"Hex"]) {
            NSCharacterSet *allowedCharacters = [NSCharacterSet characterSetWithCharactersInString:@"0123456789abcdefABCDEF"];
            return ([string rangeOfCharacterFromSet:allowedCharacters].location != NSNotFound || [string isEqualToString:@""]);
        } else if ([selectedFormat isEqualToString:@"EPC"]) {
            NSCharacterSet *allowedCharacters = [NSCharacterSet characterSetWithCharactersInString:@"0123456789,~!@#$%^&*|.<>/\\:;"];
            return ([string rangeOfCharacterFromSet:allowedCharacters].location != NSNotFound || [string isEqualToString:@""]);
        }
    }
    return YES;
}

@end

@interface RFIDReadView() <UITextFieldDelegate>

@property (nonatomic, strong) UILabel *formatLabel;       // Format label
@property (nonatomic, strong) UILabel *memoryLabel;       // Memory label
@property (nonatomic, strong) UISegmentedControl *formatSegmentedControl; // Format selection
@property (nonatomic, strong) UISegmentedControl *memorySegmentedControl; // Memory segment selection
@property (nonatomic, strong) UIButton *submitButton;                   // Submit button

@property (nonatomic, strong) UILabel *printLabel;
@property (nonatomic, strong) UISwitch *printSwitch;

- (instancetype)initWithFrame:(CGRect)frame;

@end

@implementation RFIDReadView

- (instancetype)initWithFrame:(CGRect)frame {
    self = [super initWithFrame:frame];
    if (self) {
        [self setupView];
    }
    return self;
}

- (void)setupView {
    
    self.formatLabel = [[UILabel alloc] init];
    self.formatLabel.text = @"Select the format of the content to read";
    self.formatLabel.font = [UIFont systemFontOfSize:16];
    
    self.memoryLabel = [[UILabel alloc] init];
    self.memoryLabel.text = @"Select the segment of content to read, default is EPC96";
    self.memoryLabel.font = [UIFont systemFontOfSize:16];
    
    // Format selection Segmented Control
    self.formatSegmentedControl = [[UISegmentedControl alloc] initWithItems:@[@"ASCII", @"Hex"]];
    self.formatSegmentedControl.selectedSegmentIndex = 1;

    // Memory segment selection Segmented Control
    self.memorySegmentedControl = [[UISegmentedControl alloc] initWithItems:@[@"EPC96", @"Reserved", @"EPC", @"TID", @"User"]];
    
    self.memorySegmentedControl.selectedSegmentIndex = 0;
    
    self.printSwitch = [[UISwitch alloc] init];
    self.printLabel = [[UILabel alloc] init];
    self.printLabel.text = @"Print label content simultaneously";
    
    // Submit button
    self.submitButton = [UIButton buttonWithType:UIButtonTypeSystem];
    [self.submitButton setTitle:@"Submit" forState:UIControlStateNormal];
    [self.submitButton addTarget:self action:@selector(submitData) forControlEvents:UIControlEventTouchUpInside];

    // Layout
    UIStackView *stackView = [[UIStackView alloc] initWithArrangedSubviews:@[
        self.formatLabel,
        self.formatSegmentedControl,
        self.memoryLabel,
        self.memorySegmentedControl,
        [self createStackWithLabel:self.printLabel andButton:self.printSwitch],
        self.submitButton
    ]];
    stackView.axis = UILayoutConstraintAxisVertical;
    stackView.spacing = 10;
    stackView.translatesAutoresizingMaskIntoConstraints = NO;

    [self addSubview:stackView];

    // Constraints
    [NSLayoutConstraint activateConstraints:@[
        [stackView.leadingAnchor constraintEqualToAnchor:self.leadingAnchor constant:20],
        [stackView.trailingAnchor constraintEqualToAnchor:self.trailingAnchor constant:-20],
        [stackView.topAnchor constraintEqualToAnchor:self.topAnchor constant:20],
        [stackView.bottomAnchor constraintEqualToAnchor:self.bottomAnchor constant:-20]
    ]];
}

- (UIStackView *)createStackWithLabel:(UILabel *)label andButton:(UISwitch *)switchControl {
    UIStackView *stack = [[UIStackView alloc] initWithArrangedSubviews:@[label, switchControl]];
    stack.axis = UILayoutConstraintAxisHorizontal;
    stack.spacing = 10;
    return stack;
}

- (NSString *)returnFormat:(NSInteger)index {
    switch (index) {
        case 0: return @"A";
        case 1: return @"H";
        case 2: return @"E";
        default: return @"H";
    }
}

- (NSString *)returnMemory:(NSInteger)index {
    switch (index) {
        case 0: return @"E";
        case 1: return @"0";
        case 2: return @"1";
        case 3: return @"2";
        case 4: return @"3";
        default: return @"E";
    }
}

- (void)submitData {
    NSString *blockNumber = @"2";
    if (self.onSubmit) {
        self.onSubmit(blockNumber, [self returnMemory:self.memorySegmentedControl.selectedSegmentIndex], [self returnFormat:self.formatSegmentedControl.selectedSegmentIndex],self.printSwitch.on);
    }
}

#pragma mark - UITextFieldDelegate

- (BOOL)textField:(UITextField *)textField shouldChangeCharactersInRange:(NSRange)range replacementString:(NSString *)string {
    // Limit starting block number input to digits only
    NSCharacterSet *allowedCharacters = [NSCharacterSet decimalDigitCharacterSet];
    return ([string rangeOfCharacterFromSet:allowedCharacters].location != NSNotFound || [string isEqualToString:@""]);
}

@end

