//
//  RFIDVC.m
//  PrinterDemo
//
//  Created by Apple Mac mini intel on 2024/7/19.
//

#import "RFIDVC.h"
#import "RFIDCustomView.h"
#import "XSDKPrinter.h"
#import "UIView+Toast.h"

typedef NS_ENUM(NSInteger, RFIDOperationType) {
    RFIDOperationTypeWrite,
    RFIDOperationTypeRead,
    RFIDOperationTypeSettings
};

@interface RFIDVC ()<XWIFIManagerDelegate>

@property (nonatomic, strong) TagCorrectView *tagCorrectionView;
@property (nonatomic, strong) RFIDPowerView *rfidPowerView;
@property (nonatomic, strong) RFIDWriteView *rfidWriteView;
@property (nonatomic, strong) RFIDReadView *rfidReadView;
@property (nonatomic, strong) UIView *contentView;
@property (nonatomic, strong) BottomView *bottomView;
@property (nonatomic, strong) UIScrollView *scrollView;
@property (nonatomic, strong) UISegmentedControl *segmentedControl;
@property (strong, nonatomic) XWIFIManager *wifiManager;
@end

@implementation RFIDVC

- (void)viewDidLoad {
    [super viewDidLoad];
    self.title = @"RFID";
    [self setDefaultView];
    [self selectView:RFIDOperationTypeWrite];
    _wifiManager = [XWIFIManager sharedInstance];
    _wifiManager.delegate = self;
}

- (void)printWithData:(NSData *)dataM {
    [_wifiManager writeCommandWithData:dataM];
}

- (void)selectView:(RFIDOperationType)type {
    
    self.tagCorrectionView.hidden = YES;
    self.rfidPowerView.hidden = YES;
    self.rfidWriteView.hidden = YES;
    self.rfidReadView.hidden = YES;
    
    if (type == RFIDOperationTypeWrite) {
        self.rfidWriteView.hidden = NO;
    } else if (type == RFIDOperationTypeRead) {
        self.rfidReadView.hidden = NO;
    } else if (type == RFIDOperationTypeSettings) {
        self.tagCorrectionView.hidden = NO;
        self.rfidPowerView.hidden = NO;
    }
}

- (void)touchesBegan:(NSSet<UITouch *> *)touches withEvent:(UIEvent *)event {
    [super touchesBegan:touches withEvent:event];
    [self.view endEditing:YES];
}

- (void)setDefaultView {

    self.segmentedControl = [[UISegmentedControl alloc] initWithItems:@[@"Write", @"Read", @"Settings"]];
    self.segmentedControl.selectedSegmentIndex = 0;
    self.segmentedControl.translatesAutoresizingMaskIntoConstraints = NO;
    
    [self.segmentedControl addTarget:self
                              action:@selector(segmentChanged:)
                    forControlEvents:UIControlEventValueChanged];
    
    [self.view addSubview:self.segmentedControl];
    
    self.scrollView = [[UIScrollView alloc] initWithFrame:CGRectZero];
    self.scrollView.translatesAutoresizingMaskIntoConstraints = NO;
    self.scrollView.alwaysBounceVertical = YES;

    [self.view addSubview:self.scrollView];
    
    self.contentView = [[UIView alloc] init];
    self.contentView.translatesAutoresizingMaskIntoConstraints = NO;
    [self.scrollView addSubview:self.contentView];
    
    __weak typeof(self) weakSelf = self;

    
    // Initialize BottomView
    self.bottomView = [[BottomView alloc] initWithFrame:CGRectZero];
    self.bottomView.translatesAutoresizingMaskIntoConstraints = NO;
    //self.bottomView.backgroundColor = [UIColor yellowColor];
    [self.view addSubview:self.bottomView];
    
    /// Read
    self.rfidReadView = [[RFIDReadView alloc] initWithFrame:self.view.bounds];
    self.rfidReadView.translatesAutoresizingMaskIntoConstraints = NO;
    self.rfidReadView.onSubmit = ^(NSString *blockNumber, NSString *memorySegment, NSString *formatSegment,BOOL isPrint) {
        NSLog(@"blockNumber: %@", blockNumber);
        NSLog(@"memorySegment: %@", memorySegment);
        NSLog(@"formatSegment: %@", formatSegment);
        
        XZPLCommand *cmd = [[XZPLCommand alloc] init];
        [cmd start];
        if (isPrint) {
            [cmd rfidDefineFontAtX:70 y:100 fontName:kFNT_0 rotation:ZPLRotation0 sizeW:40 sizeH:40];
        }
        [cmd rfidRead:formatSegment beginNum:nil byteCount:@"8" memoryBank:memorySegment isPrint:isPrint];
        [cmd end];
        [weakSelf printWithData:[cmd getCommand]];
    };
    [self.contentView addSubview:self.rfidReadView];
    
    /// Settings
    self.tagCorrectionView = [[TagCorrectView alloc] initWithFrame:CGRectZero];
    self.tagCorrectionView.onSend = ^{
        XZPLCommand *cmd = [[XZPLCommand alloc] init];
        [cmd start];
        [cmd rfidCalibration];
        [cmd end];
        [weakSelf printWithData:[cmd getCommand]];
    };
    self.tagCorrectionView.translatesAutoresizingMaskIntoConstraints = NO;
    [self.contentView addSubview:self.tagCorrectionView];
    
    self.rfidPowerView = [[RFIDPowerView alloc] initWithFrame:CGRectZero];
    self.rfidPowerView.translatesAutoresizingMaskIntoConstraints = NO;
    self.rfidPowerView.onSubmit = ^(NSString *readPower, NSString *writePower) {
        NSLog(@"readPower: %@, writePower: %@", readPower, writePower);
        if (readPower.intValue <= 30 && writePower.intValue <= 30) {
            XZPLCommand *cmd = [[XZPLCommand alloc] init];
            [cmd start];
            [cmd rfidReadPower:readPower writePower:writePower];
            [cmd end];
            [weakSelf printWithData:[cmd getCommand]];
        } else {
            [weakSelf.view makeToast:@"readPower value or writePower value cannot exceed 30." duration:1.f position:CSToastPositionCenter];
        }
    };
    [self.contentView addSubview:self.rfidPowerView];

    self.rfidWriteView = [[RFIDWriteView alloc] initWithFrame:CGRectZero];
    self.rfidWriteView.translatesAutoresizingMaskIntoConstraints = NO;
    
    self.rfidWriteView.onSubmit = ^(NSString *content, NSString *blockNumber, NSString *memorySegment, NSString *formatSegment,BOOL isRead,BOOL isPrint) {
        
        XZPLCommand *cmd = [[XZPLCommand alloc] init];
        [cmd start];
        
        if ([formatSegment isEqualToString:@"A"]) {
            [cmd rfidWrite:content format:formatSegment beginNum:nil byteCount:nil memoryBank:nil];
        } else if ([formatSegment isEqualToString:@"H"]) {
            if ([weakSelf byteCountFromHexString:content]) {
                [cmd rfidWrite:content format:formatSegment beginNum:nil byteCount:[NSString stringWithFormat:@"%lu",(unsigned long)[weakSelf byteCountFromHexString:content]] memoryBank:memorySegment];
            } else {
                [weakSelf.view makeToast:@"Invalid hex string" duration:1.f position:CSToastPositionCenter];
            }
        }
        if (isRead) {
            [cmd rfidRead:formatSegment beginNum:nil byteCount:@"8" memoryBank:memorySegment isPrint:isPrint];
        }
        [cmd end];
        [weakSelf printWithData:[cmd getCommand]];
       };
    
    [self.contentView addSubview:self.rfidWriteView];
    
    
    // Layout
    [NSLayoutConstraint activateConstraints:@[
        [self.segmentedControl.topAnchor constraintEqualToAnchor:self.view.safeAreaLayoutGuide.topAnchor constant:16],
        [self.segmentedControl.leadingAnchor constraintEqualToAnchor:self.view.leadingAnchor constant:16],
        [self.segmentedControl.trailingAnchor constraintEqualToAnchor:self.view.trailingAnchor constant:-16],
        
        [self.bottomView.leadingAnchor constraintEqualToAnchor:self.view.leadingAnchor],
        [self.bottomView.trailingAnchor constraintEqualToAnchor:self.view.trailingAnchor],
        [self.bottomView.bottomAnchor constraintEqualToAnchor:self.view.safeAreaLayoutGuide.bottomAnchor],
        [self.bottomView.heightAnchor constraintEqualToConstant:300],
        
        [self.scrollView.topAnchor constraintEqualToAnchor:self.segmentedControl.bottomAnchor constant:16],
        [self.scrollView.leadingAnchor constraintEqualToAnchor:self.view.leadingAnchor],
        [self.scrollView.trailingAnchor constraintEqualToAnchor:self.view.trailingAnchor],
        [self.scrollView.bottomAnchor constraintEqualToAnchor:self.bottomView.topAnchor],
        
        [self.contentView.leadingAnchor constraintEqualToAnchor:self.scrollView.leadingAnchor],
        [self.contentView.trailingAnchor constraintEqualToAnchor:self.scrollView.trailingAnchor],
        [self.contentView.topAnchor constraintEqualToAnchor:self.scrollView.topAnchor],
        [self.contentView.bottomAnchor constraintEqualToAnchor:self.scrollView.bottomAnchor],
        [self.contentView.heightAnchor constraintEqualToConstant:400],
        
        /// Write
        [self.rfidWriteView.topAnchor constraintEqualToAnchor:self.scrollView.topAnchor constant:0],
        [self.rfidWriteView.leadingAnchor constraintEqualToAnchor:self.scrollView.leadingAnchor],
        [self.rfidWriteView.trailingAnchor constraintEqualToAnchor:self.scrollView.trailingAnchor],
        [self.rfidWriteView.widthAnchor constraintEqualToConstant:self.view.frame.size.width],
        [self.rfidWriteView.heightAnchor constraintEqualToConstant:380],

        /// Read
        [self.rfidReadView.topAnchor constraintEqualToAnchor:self.scrollView.topAnchor constant:0],
        [self.rfidReadView.leadingAnchor constraintEqualToAnchor:self.scrollView.leadingAnchor],
        [self.rfidReadView.trailingAnchor constraintEqualToAnchor:self.scrollView.trailingAnchor],
        [self.rfidReadView.widthAnchor constraintEqualToConstant:self.view.frame.size.width],
        [self.rfidReadView.heightAnchor constraintEqualToConstant:220],
        
        /// Settings
        [self.tagCorrectionView.topAnchor constraintEqualToAnchor:self.scrollView.topAnchor constant:0],
        [self.tagCorrectionView.leadingAnchor constraintEqualToAnchor:self.scrollView.leadingAnchor],
        [self.tagCorrectionView.trailingAnchor constraintEqualToAnchor:self.scrollView.trailingAnchor],
        [self.tagCorrectionView.widthAnchor constraintEqualToConstant:self.view.frame.size.width],
        [self.tagCorrectionView.heightAnchor constraintEqualToConstant:40],
        
        [self.rfidPowerView.topAnchor constraintEqualToAnchor:self.tagCorrectionView.bottomAnchor constant:0],
        [self.rfidPowerView.leadingAnchor constraintEqualToAnchor:self.scrollView.leadingAnchor],
        [self.rfidPowerView.trailingAnchor constraintEqualToAnchor:self.scrollView.trailingAnchor],
        [self.rfidPowerView.widthAnchor constraintEqualToConstant:self.view.frame.size.width],
        [self.rfidPowerView.heightAnchor constraintEqualToConstant:160],

    ]];
}

- (NSUInteger)byteCountFromHexString:(NSString *)hexString {
    
    if (hexString.length % 2 != 0) {
        // If the string length is not even, return 0 or throw an exception
        NSLog(@"Invalid hex string");
        return 0;
    }

    NSMutableData *data = [NSMutableData data];
    for (NSUInteger i = 0; i < hexString.length; i += 2) {
        NSString *byteString = [hexString substringWithRange:NSMakeRange(i, 2)];
        unsigned int byteValue;
        [[NSScanner scannerWithString:byteString] scanHexInt:&byteValue];
        [data appendBytes:&byteValue length:1];
    }

    return data.length;
}

- (void)segmentChanged:(UISegmentedControl *)sender {
    NSInteger selectedIndex = sender.selectedSegmentIndex;
    switch (selectedIndex) {
        case 0:
            [self selectView:RFIDOperationTypeWrite];
            break;
        case 1:
            [self selectView:RFIDOperationTypeRead];
            break;
        case 2:
            [self selectView:RFIDOperationTypeSettings];
            break;
        default:
            break;
    }
}

#pragma mark - XWIFIManagerDelegate

- (void)xWifiDisconnectWithError:(NSError *)error {
    [self.view makeToast:@"disconnect.." duration:1.f position:CSToastPositionCenter];
    [self.navigationController popViewControllerAnimated:YES];
}

- (void)xWifiWriteValueWithTag:(long)tag {
    NSLog(@"xWifiWriteValueWithTag:%ld", tag);
}

- (void)xWifiReceiveValueForData:(NSData *)data {
    NSLog(@"wifi receive value:%@", data);
    NSString *receivedString = [[NSString alloc] initWithData:data encoding:NSASCIIStringEncoding];
    if (receivedString) {
        NSString *currentText = self.bottomView.textView.text;
        NSString *updatedText = [currentText stringByAppendingFormat:@"%@\n", receivedString];
        dispatch_async(dispatch_get_main_queue(), ^{
            self.bottomView.textView.text = updatedText;
        });
    } else {
        NSLog(@"Failed to decode data into ASCII string.");
    }
}


@end
